@@ -3,10 +3,19 @@ import json
 import logging
 import operator
 import requests
+import urllib
 
+from ... import exceptions as main_exceptions
 from ...metadata import Metadata
-from ...numbers import from_lovelaces
-from ...simpletypes import AssetID, Balance
+from ...numbers import from_lovelaces, to_lovelaces
+from ...simpletypes import (
+    AssetID,
+    Balance,
+    StakePoolInfo,
+    StakePoolStatus,
+    StakeRewardMetrics,
+    StakingStatus,
+)
 from ...transaction import Transaction
 from . import exceptions
 from . import serializers
@@ -25,6 +34,16 @@ class WalletREST(object):
     base_url = None
     timeout = 10
 
+    ERR2EXCEPTION = {
+        403: {
+            "pool_already_joined": main_exceptions.PoolAlreadyJoined,
+            "non_null_rewards": main_exceptions.NonNullRewards,
+        },
+        500: {
+            "created_invalid_transaction": exceptions.CreatedInvalidTransaction,
+        },
+    }
+
     def __init__(self, protocol="http", host="localhost", port=8090):
         self.base_url = "{protocol}://{host}:{port}/v2/".format(
             protocol=protocol, host=host, port=port
@@ -48,7 +67,7 @@ class WalletREST(object):
         if rsp.status_code != 204:  # if content exists
             result = rsp.json(parse_float=Decimal)
             _ppresult = json.dumps(
-                result, cls=JSONWithDecimalEncoder, indent=2, sort_keys=True
+                rsp.json(), cls=JSONWithDecimalEncoder, indent=2, sort_keys=True
             )
             _log.debug(u"Result:\n{result}".format(result=_ppresult))
         else:
@@ -56,8 +75,26 @@ class WalletREST(object):
             _log.debug(u"No result (HTTP 204)")
         if rsp.status_code == 400:
             raise exceptions.BadRequest(result["message"], result=result)
+        if rsp.status_code == 403:
+            try:
+                raise self.ERR2EXCEPTION[rsp.status_code][result["code"]](
+                    result["message"]
+                )
+            except KeyError:
+                pass
+            raise exceptions.RESTServerError(result.get("message", "* NO MESSAGE *"))
         if rsp.status_code == 404:
             raise exceptions.NotFound(result["message"], result=result)
+        if rsp.status_code == 500:
+            try:
+                raise self.ERR2EXCEPTION[rsp.status_code][result["code"]](
+                    result["message"], result=result
+                )
+            except KeyError:
+                pass
+            raise exceptions.RESTServerError(
+                result.get("message", "* NO MESSAGE *"), result=result
+            )
         return result
 
     def wallet_ids(self):
@@ -177,6 +214,10 @@ class WalletREST(object):
             outputs=outputs,
             local_inputs=local_inputs,
             local_outputs=local_outputs,
+            withdrawals=[
+                (serializers.get_amount(w["amount"]), w["stake_address"])
+                for w in txd.get("withdrawals", [])
+            ],
             metadata=metadata,
         )
 
@@ -195,9 +236,10 @@ class WalletREST(object):
             )
         ]
 
-    def transfer(self, wid, destinations, metadata, ttl, passphrase):
+    def transfer(self, wid, destinations, metadata, allow_withdrawal, ttl, passphrase):
         data = {
             "passphrase": passphrase,
+            "withdrawal": "self" if allow_withdrawal else None,
             "payments": [
                 {
                     "address": str(address),
@@ -237,3 +279,99 @@ class WalletREST(object):
             serializers.get_amount(feedata["estimated_min"]),
             serializers.get_amount(feedata["estimated_max"]),
         )
+
+    def _stakepoolinfo(self, pooldata, stake):
+        retirement = (
+            serializers.get_epoch(pooldata["retirement"])
+            if "retirement" in pooldata and "epoch_number" in pooldata["retirement"]
+            else None
+        )
+        status = (
+            StakePoolStatus.DELISTED
+            if "flags" in pooldata and "delisted" in pooldata["flags"]
+            else StakePoolStatus.RETIRING
+            if retirement
+            else StakePoolStatus.ACTIVE
+        )
+        if "metadata" in pooldata:
+            ticker = pooldata["metadata"]["ticker"]
+            name = pooldata["metadata"]["name"]
+            description = pooldata["metadata"]["description"]
+            homepage = pooldata["metadata"]["homepage"]
+        else:
+            ticker = name = description = homepage = None
+        rewards = StakeRewardMetrics(
+            serializers.get_amount(pooldata["metrics"]["non_myopic_member_rewards"]),
+            stake,
+        )
+        return StakePoolInfo(
+            pooldata["id"],
+            status,
+            ticker,
+            name,
+            description,
+            homepage,
+            rewards,
+            serializers.get_amount(pooldata["cost"]),
+            serializers.get_percent(pooldata["margin"]),
+            serializers.get_amount(pooldata["pledge"]),
+            serializers.get_percent(pooldata["metrics"]["relative_stake"]),
+            pooldata["metrics"]["saturation"],
+            pooldata["metrics"]["produced_blocks"]["quantity"],
+            retirement,
+        )
+
+    def stake_pools(self, wid, stake):
+        urldata = {"stake": to_lovelaces(stake)}
+        poolsdata = self.raw_request(
+            "GET", "?".join(("stake-pools", urllib.parse.urlencode(urldata)))
+        )
+        return [self._stakepoolinfo(pool, stake) for pool in poolsdata]
+
+    def _stakingstatus(self, data):
+        return StakingStatus(
+            serializers.get_stakingstatus(data["status"]),
+            data["target"] if "target" in data else None,
+            serializers.get_epoch(data["changes_at"]) if "changes_at" in data else None,
+        )
+
+    def staking_status(self, wid):
+        sdata = self.raw_request("GET", "wallets/{:s}".format(wid))["delegation"]
+        active = sdata["active"]
+        return (
+            self._stakingstatus(active),
+            [self._stakingstatus(ss) for ss in sdata["next"]],
+        )
+
+    def stake(self, wid, pool_id, passphrase):
+        txdata = self.raw_request(
+            "PUT",
+            "stake-pools/{:s}/wallets/{:s}".format(pool_id, wid),
+            {"passphrase": passphrase},
+        )
+        return self._txdata2tx(txdata, addresses=self._addresses_set(wid))
+
+    def unstake(self, wid, passphrase):
+        txdata = self.raw_request(
+            "DELETE",
+            "stake-pools/*/wallets/{:s}".format(wid),
+            {"passphrase": passphrase},
+        )
+        return self._txdata2tx(txdata, addresses=self._addresses_set(wid))
+
+    def utxo_stats(self, wid):
+        sdata = self.raw_request(
+            "GET",
+            "wallets/{:s}/statistics/utxos".format(wid),
+            {},
+        )
+        return (
+            serializers.get_amount(sdata["total"]),
+            {
+                from_lovelaces(int(lvl)): num
+                for (lvl, num) in sorted(
+                    sdata["distribution"].items(), key=operator.itemgetter(0)
+                )
+            },
+            sdata["scale"],
+        )
