@@ -1,7 +1,7 @@
 from ... import exceptions
 
 
-class WalletRESTException(exceptions.BackendException):
+class WalletRESTException(exceptions.BackendException, exceptions.WalletException):
     def __init__(self, *args, **kwargs):
         self.result = kwargs.pop("result", None)
         super(WalletRESTException, self).__init__(*args, **kwargs)
@@ -13,3 +13,11 @@ class BadRequest(WalletRESTException):
 
 class NotFound(WalletRESTException):
     pass
+
+
+class RESTServerError(WalletRESTException):
+    pass
+
+
+class CreatedInvalidTransaction(WalletRESTException, exceptions.TransactionException):
+    pass
