@@ -1,4 +1,3 @@
-import collections
 from decimal import Decimal
 from warnings import warn
 
@@ -6,7 +5,13 @@ LOVELACE = Decimal("0.000001")
 
 
 def to_lovelaces(amount):
-    """Convert ADA decimal to integer of Lovelaces."""
+    """
+    Convert ADA to Lovelaces.
+
+    :param Decimal,int amount:    the amount of ADA
+    :rtype: :class:`int`
+
+    """
     if not isinstance(amount, (Decimal, int, float)):
         raise ValueError(
             "Amount '{}' doesn't have numeric type. Only Decimal, int and "
@@ -24,12 +29,22 @@ def to_lovelaces(amount):
 
 
 def from_lovelaces(amount):
-    """Convert amount of Lovelaces to ADA decimal."""
+    """
+    Convert Lovelaces to ADA.
+
+    :param int amount:  the amount of Lovelaces
+    :rtype: :class:`Decimal`
+    """
     return (Decimal(amount) * LOVELACE).quantize(LOVELACE)
 
 
 def as_ada(amount):
-    """Return the amount rounded to maximal ADA precision."""
+    """
+    Return the amount rounded to maximal ADA precision.
+
+    :param Decimal,int amount:  the amount to be sanitized
+    :rtype: :class:`Decimal` with 6 decimal places precision
+    """
     if isinstance(amount, float):
         warn(
             "as_ada() received amount of float type ({:f}). It is STRONGLY DISCOURAGED "
@@ -39,8 +54,3 @@ def as_ada(amount):
             RuntimeWarning,
         )
     return Decimal(amount).quantize(LOVELACE)
-
-
-BlockPosition = collections.namedtuple(
-    "BlockPosition", ["epoch", "slot", "absolute_slot"]
-)
